/*  SAP DB ODBC example, 2000-09-26

    Progname      odbcexample.c

    Build:        make odbcexample

    Description:  Creates a table, inserts some values and read them again

    Note:         For convinience, not all return codes of the ODBC functions
                  are evaluated.
*/

#include <stdio.h>
#include <string.h>
#include "WINDOWS.H"
#include "sql.h"
#include "sqlext.h"

#define MAX_NAME_LEN 50
#define MAX_STMT_LEN 300

int printSQLError (SQLHDBC hdbc, SQLHSTMT hstmt);

typedef struct ConnectArgsT {
    char * username;
    char * password;
    char * dbname;
    char * host;
} ConnectArgsT;


static void parseArgs (ConnectArgsT * connectArgs, int argc, char **argv)
{
    /*
     * setting defaults for demo database
     */
    connectArgs->username = "TEST";
    connectArgs->password = "TEST";
    connectArgs->dbname = "TST";
    connectArgs->host = "localhost";
    /*
     * use values from command line
     */
    if (argc > 4) {
        connectArgs->host = argv [4];
    }
    if (argc > 3) {
        connectArgs->dbname = argv [3];
    }
    if (argc > 2) {
        connectArgs->password = argv [2];
    }
    if (argc > 1) {
        connectArgs->username = argv [1];
    }
}

int main (int argc, char **argv)
{
  ConnectArgsT connectArgs;
  SQLHENV   henv;
  SQLHDBC   hdbc;
  SQLHSTMT  hstmt;

  UCHAR   result[MAX_NAME_LEN + 1];
  UCHAR   select[MAX_STMT_LEN];
  UCHAR   server[MAX_STMT_LEN];
  SDWORD  resultLen;
  RETCODE rc;

  parseArgs (&connectArgs, argc, argv);

  sprintf (server, "%s:%s", connectArgs.host, connectArgs.dbname);

/* Allocate environment, connection, and statement handle  */
  SQLAllocHandle (SQL_HANDLE_ENV, SQL_NULL_HANDLE, &henv);
  SQLAllocHandle (SQL_HANDLE_DBC, henv, &hdbc);

/* Connect to data source                                  */
  rc = SQLConnect (hdbc, server, SQL_NTS, connectArgs.username,
          SQL_NTS, connectArgs.password, SQL_NTS);
  if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
    return (printSQLError (hdbc, SQL_NULL_HSTMT));
  SQLAllocHandle (SQL_HANDLE_STMT, hdbc, &hstmt);

/* Select from table */
  strcpy (select, "SELECT 'Hello, SAP DB' from dual");
  if (SQLExecDirect (hstmt, select, SQL_NTS) != SQL_SUCCESS)
    return (printSQLError (hdbc, hstmt));

/* Bind columns of the result set. */
  SQLBindCol (hstmt, 1, SQL_C_CHAR, result, (SDWORD)sizeof(result), &resultLen);

/* fetch all rows and print them */
  while (1)
    {
    rc = SQLFetch (hstmt);
    if (rc == SQL_SUCCESS || rc == SQL_SUCCESS_WITH_INFO)
      printf ("%s\n", result);
    else if (rc == SQL_NO_DATA)
      break;
    else
      {
      printSQLError (hdbc, hstmt);
      break;
      }
    }


/* Disconnect from the data source and free all handles. */
  SQLDisconnect (hdbc);
  SQLFreeHandle (SQL_HANDLE_STMT, hstmt);
  SQLFreeHandle (SQL_HANDLE_DBC,  hdbc);
  SQLFreeHandle (SQL_HANDLE_ENV,  henv);

  return (0);
}

int printSQLError (SQLHDBC hdbc, SQLHSTMT hstmt)
{
   UCHAR  sqlstate[10];
   SDWORD sqlcode;
   UCHAR  errortxt[MAX_STMT_LEN+1];
   SWORD  usederrortxt;

   SQLError (SQL_NULL_HENV, hdbc, hstmt, sqlstate, &sqlcode, errortxt,
	     MAX_STMT_LEN, &usederrortxt);

   printf (" SQL state: %s\n", sqlstate);
   printf (" SQL code:  %ld\n", sqlcode);
   printf (" SQL Errortext:\n%s\n\n", errortxt);
   return (0);
}
